%define ALD_Ver		1.2.4

Name:         pcre

License:      BSD
Summary:      Perl Compatible Expressions Library
Group:        System Environment/Libraries
Version:      8.38
Release:      0

Source: %{name}-%{version}.tar.xz
Source1: CROSS.sh
Source2: pcre3_8.38-3.1.debian.tar.gz

%define DestDir		/text

%define USE_LIB_CPP 1
Provides: libpcre.so.0

# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64/armv6l/win32 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mfpu=vfpv3-d16 -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1
#------------------ CROSS DIR --------------------
%if %{_CROSS}

%if %{_os} == "mingw"
ERROR: target=%{_os}
%define target %{_arch}-%{_vendor}-%{_os}%{__isa_bits}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif
%define target_  %{target}-

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/bin

%define __os_install_post %{nil}

%define cross_bindir	%{csysdir}%{_bindir}
%define cross_sbindir	%{csysdir}%{_sbindir}
%define cross_libdir	%{csysdir}%{_libdir}
%define cross_includedir %{csysdir}%{_includedir}
%define cross_datadir	%{csysdir}%{_datadir}
%define cross_infodir	%{csysdir}%{_infodir}
%define cross_mandir	%{csysdir}%{_mandir}
%define cross_defaultdocdir %{csysdir}%{_defaultdocdir}
%define cross_docdir	%{cross_defaultdocdir}/%{name}

%define CrossDestDir	/Cross

%define _srcrpmdir	%{_topdir}/SRPMS/tmp

%else  #_CROSS

%define csysbindir	%{_prefix}/bin
%define CrossDestDir	%{nil}
%define _srcrpmdir	%{_topdir}/SRPMS/%{ALD_Ver}

%define target %{_build}
%define target_  %{nil}

%endif # !_CROSS

%define docdir %{_defaultdocdir}/%{name}

# ------------------- description ------------------------------------
%description
PCRE is a library of functions to support regular expressions whose syntax
and semantics are as close as possible to those of the Perl 5 language.
#-------------------------------
%package libcpp
Summary:      C++ Libraries
Group:        System Environment/Libraries
%description libcpp
C++ Libraries
#-------------------------------
%package devel
Summary:      Libraries for the C compiler
Group:        Development/Libraries
%description devel
These libraries are needed to compile C code.
#-------------------------------
%package devel-cpp
Summary:      Libraries for the C++ compiler
Group:        Development/Libraries
%description devel-cpp
These libraries are needed to compile C++ code.
#-------------------------------
%package devel-doc
Summary:      Libraries doc for %{name}
Group:        Documentation
%description devel-doc
API doc for %{name}
#-------------------------------
%package doc
Summary:      doc for %{name}
Group:        Documentation
%description doc
doc for %{name}

#-------------------------------
%package %{TO_ARCH}
Summary:      %{name} Cross-%{TO_ARCH} Libraries
Group:        Development/Libraries
%description %{TO_ARCH}
%{name} Cross-%{TO_ARCH} Libraries
# --------------------------------------
%package %{TO_ARCH}-staticlib
Summary:      %{name} Cross-%{TO_ARCH} static Libraries
Group:        Development/Libraries
%description %{TO_ARCH}-staticlib
%{name} Cross-%{TO_ARCH} static Libraries

# ------------------- setup ------------------------------------------
%define DBP		debian/patches/

%Prep
%setup -q -n %{name}-%{version}
tar -xzf %SOURCE2

%if 1
patch -p1<%{DBP}PCRE6_compatible_API.patch
patch -p1<%{DBP}pcreposix.patch
patch -p1<%{DBP}pcre_info.patch
patch -p1<%{DBP}pcregrep.1-patch
#patch -p1<%{DBP}soname.patch
patch -p1<%{DBP}no_jit_x32_powerpcspe.patch
patch -p1<%{DBP}Disable_JIT_on_sparc64.patch
patch -p1<%{DBP}CVE-2016-1283.patch
%endif

# ------------------- Build ------------------------------------------
%Build
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

GCC_VER_STR=`gcc -dumpversion`

%if %{_arch} == "i386"
OPTFLAGS="-Os -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
OPT47=`case $GCC_VER_STR in 4.[789].*) echo "-fno-asynchronous-unwind-tables" ;; esac`
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-Os"
OPT47=`case $GCC_VER_STR in 4.[789].*) echo "-fno-asynchronous-unwind-tables" ;; esac`
%endif
%if %{_arch} == "arm"
OPTFLAGS="-Os -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%endif

CFLAGS="$OPTFLAGS $OPT47 -ffunction-sections -fdata-sections"
CXXFLAGS="$OPTFLAGS $OPT47"
LDFLAGS="-Wl,--gc-sections"

%configure \
	--build=%{_build} \
%if %{_CROSS}
	--host=%{target} \
%endif
	--enable-shared \
	--enable-static \
	--disable-libtool-lock \
	--disable-pcregrep-jit \
	--enable-unicode-properties \
%if %{USE_LIB_CPP}
	--enable-cpp \
%else
	--disable-cpp \
%endif
	--enable-pcregrep-libz \
	--enable-pcregrep-libbz2 \
	&>FConfigure_log.txt


make &>FMake_log.txt
#make V=1 &>FMake_log.txt

# ------------------- Install ----------------------------------------
%Install
%if %{_CROSS}
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=$PATH:/opt/perl/bin
%endif

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}

make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt

rm $RPM_BUILD_ROOT%{_libdir}/*.la

pushd $RPM_BUILD_ROOT%{_libdir}
ln -s libpcre.so.1.2.6 libpcre.so.0
popd


%if %{_CROSS}
TARGET_PRFIX=%{_prefix}
TARGET_CROSS_PRFIX=%{csysdir}
TARGET_NAME=%{target}
MKCROSSDIR=%{BUILD_LOCAL_CROSS}
export TARGET_PRFIX TARGET_CROSS_PRFIX TARGET_NAME MKCROSSDIR
%SOURCE1
%if %{BUILD_LOCAL_CROSS}
mkdir -p $RPM_BUILD_ROOT%{csysdir}/bin
mv $RPM_BUILD_ROOT%{cross_bindir}/pcre-config $RPM_BUILD_ROOT%{csysdir}/bin
rm -r $RPM_BUILD_ROOT%{cross_bindir}
rm -r $RPM_BUILD_ROOT%{cross_docdir}/html
rm $RPM_BUILD_ROOT%{cross_docdir}/ChangeLog
rm $RPM_BUILD_ROOT%{cross_docdir}/NEWS
rm $RPM_BUILD_ROOT%{cross_docdir}/pcre*
rm -r $RPM_BUILD_ROOT%{cross_mandir}
%endif #BUILD_LOCAL_CROSS
%endif #_CROSS

%if !%{_CROSS}
if test -f /opt/aldccp/bin/packages_idx; then
  /opt/aldccp/bin/packages_idx -p %{name} -v %{version} -r %{DestDir} $RPM_BUILD_ROOT
fi
%endif

# ------------------- Clean ------------------------------------------
%Clean 

DE=%{version}-%{release}.%{TO_ARCH}.rpm
DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{DestDir}
DEC=%{version}-%{release}.%{_build_cpu}.rpm
DDC=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
  mv %{name}-devel-$SE $DD/%{name}-devel-$DE
  mv %{name}-devel-doc-$SE $DD/%{name}-devel-doc-$DE
  mv %{name}-doc-$SE $DD/%{name}-doc-$DE

%if %{USE_LIB_CPP}
  mv %{name}-devel-cpp-$SE $DD/%{name}-devel-cpp-$DE
  mv %{name}-libcpp-$SE $DD/%{name}-libcpp-$DE
%endif #USE_LIB_CPP

%if %{BUILD_LOCAL_CROSS}
  mkdir -p $DDC
  mv %{name}-%{TO_ARCH}-$SE $DDC/%{name}-%{TO_ARCH}-$DEC
%endif #BUILD_LOCAL_CROSS
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
  TMP_TO=$DD/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}.%{TO_ARCH}
  mkdir -p $tmp
  cp -r usr $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%if %{BUILD_LOCAL_CROSS}
  TMP_TO=$DDC/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{TO_ARCH}-%{version}-%{release}.%{_build_cpu}
  mkdir -p $tmp
  cp -r cross $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%endif
popd
%endif  #BUILD_TAR

# ------------------- Files ------------------------------------------

%files
%{_bindir}/pcregrep
%{_bindir}/pcretest
%{_libdir}/libpcre.so.*
%{_libdir}/libpcreposix.so.*

%{docdir}/AUTHORS
%{docdir}/COPYING
%{docdir}/LICENCE
%{_mandir}/man1


%files devel
%{_bindir}/pcre-config
%{_libdir}/pkgconfig/libpcre.pc
%{_libdir}/pkgconfig/libpcreposix.pc
%{_includedir}/pcre.h
%{_includedir}/pcreposix.h
%{_libdir}/libpcre.so
%{_libdir}/libpcre.a
%{_libdir}/libpcreposix.so
%{_libdir}/libpcreposix.a

%if %{USE_LIB_CPP}
%files libcpp
%{_libdir}/libpcrecpp.so.*

%files devel-cpp
%{_libdir}/libpcrecpp.so
%{_libdir}/libpcrecpp.a
%{_libdir}/pkgconfig/libpcrecpp.pc
%{_includedir}/pcre_scanner.h
%{_includedir}/pcre_stringpiece.h
%{_includedir}/pcrecpp.h
%{_includedir}/pcrecpparg.h
%endif #USE_LIB_CPP

%files devel-doc
%{docdir}/pcre-config.txt
%{docdir}/pcre.txt
%{_mandir}/man3

%files doc
%{docdir}/html
%{docdir}/NEWS
%{docdir}/README
%{docdir}/ChangeLog
%{docdir}/pcregrep.txt
%{docdir}/pcretest.txt

%if %{BUILD_LOCAL_CROSS}

%files %{TO_ARCH}
%{csysdir}/bin
%{cross_libdir}/*.so.*
%{cross_libdir}/*.so
%{cross_libdir}/*.a
%{cross_includedir}
%{cross_docdir}
%endif #!BUILD_LOCAL_CROSS
